@@ -16,7 +16,7 @@
 DSET extended from DSNT (soft-argmax) operations for use in PyTorch computation graphs.
 """
 
-__version__ = "0.1.5"
+__version__ = "0.1.6"
 __author__ = 'Albert Garcia'
 __credits__ = 'SnT - Interdisciplinary Centre for Security, Reliability and Trust'
 
@@ -446,11 +446,12 @@ def construct_covariance_matrices(var, cov):
     return torch.stack(all_sigmas, dim=0)
 
 
-def covariance_matrices_to_parameters(sigmas):
+def covariance_matrices_to_parameters(sigmas, sort_ellipse_axis=True):
     """Compute the axes and rotation from all covariance matrices of a batch.
 
     Args:
         sigmas (torch.Tensor): Covariances matrices with shape [B, E, 2, 2]
+        sort_ellipse_axis (bool, default=True): Whether to sort the ellipse axes lengths in descending order when computing them through eigenvalue decomposition
 
     Returns:
         Tensor containing all the extracted parameters (axis_1, axis_2, theta [rads]) with shape [B, E, 3]
@@ -465,8 +466,21 @@ def covariance_matrices_to_parameters(sigmas):
         for location_num in range(locations):
             D, R = torch.eig(sigmas[batch_num, location_num, :, :], eigenvectors=True)
             axes = torch.sqrt(torch.abs(D))[:,0] # First column is the real part, second the imaginary part
-            cos, sin = R[0,0], R[1,0]
-            theta = torch.atan2(sin, cos).unsqueeze(0)
+            
+            if sort_ellipse_axis and axes[0] < axes[1]:
+                axes[[0, 1]] = axes[[1, 0]] # swap ellipse axes
+                R[:, [0, 1]] = R[:, [1, 0]] # swap columns
+                if torch.linalg.det(R) < 0: R[:,0] = -R[:,0] # if determinant -1 then swap sign of first column
+                cos, sin = R[0,0], R[1,0]
+                theta = torch.atan2(sin, cos)
+                if theta > np.pi/2 or theta < -np.pi/2: # if theta outside [-pi/2, pi/2] then change sign of R and recompute theta
+                    R = -R
+                    cos, sin = R[0,0], R[1,0]
+                    theta = torch.atan2(sin, cos)
+                theta = theta.unsqueeze(0)
+            else:
+                cos, sin = R[0,0], R[1,0]
+                theta = torch.atan2(sin, cos).unsqueeze(0)
             batch_params.append( torch.cat( (axes, theta) ) )
         
         batch_params =  torch.stack(batch_params, dim=0)
